#include once <print42.bas>


' Takes room number and returns address of room data
' Saves in variable
SUB FASTCALL MapSetRoom(roomno AS UBYTE)
  ASM
  PUSH IX
  LD IX, RoomPointer

  ; Lookup room address in vector table
  LD H, 0
  LD L, A	; A register = room
  ADD HL, HL
  LD DE, RoomLookupTable
  ADD HL, DE

  ; Read address of room info from vector table
  LD A, (HL)
  INC HL
  LD H, (HL)
  LD L, A

  ; Save in room pointer
  LD (IX+0), L
  LD (IX+1), H

  ; Get and Save Room Type and Colour
  LD A, (HL)
  LD B, A
  AND 15
  LD (IX+2), A    ; (IX+2)=RoomType
  LD A, B
  RRA
  RRA
  RRA
  RRA
  AND 7
  LD (IX+3), A    ; (IX+3)=RoomColour

  ; Count doors
  LD C, 0
  INC HL  ; Move past "Room Type and Colour"

LOCAL loop1
loop1:

  LD A, (HL)
  INC HL
  INC A   ; 255->0 (255 = end of doors)
  JR Z, skip2
  INC C
  INC HL  ; Move past "goto room #"
  JR loop1

LOCAL skip2
skip2:

  ; Store door Count
  LD (IX+4),C     ; (IX+4)=RoomDoorCount

  POP IX
  END ASM
END SUB

FUNCTION FASTCALL MapGetRoomType() AS UBYTE
  ASM
  LD A, (RoomType)
  END ASM
END FUNCTION

FUNCTION FASTCALL MapGetRoomColour() AS UBYTE
  ASM
  LD A, (RoomColour)
  END ASM
END FUNCTION

FUNCTION FASTCALL MapGetDoorCount() AS UBYTE
  ASM
  LD A, (RoomDoorCount)
  END ASM
END FUNCTION

SUB FASTCALL MapSetDoorNo(doorno AS UBYTE)
  ASM
  PUSH IX
  LD IX, RoomPointer

  ; Door(i) info is at: (RoomPointer) + 1 + 2 * i
  ADD A, A  ; 2*i
  INC A     ; 2*i+1
  LD HL, (RoomPointer)
  LD D, 0
  LD E, A
  ADD HL, DE
  LD (IX+5), L      ; Save current door pointer
  LD (IX+6), H      ; ALL OKAY TO HERE
  LD A, (HL)        ; Read Door type
  LD (IX+7), A     ; (IX+10) = DoorType

; Door Type
;  (bits 0-1); Side (n,s,e,w or nw,ne,se,sw)
;  (bits 2-5); Type (0=door,1=door2,2=red,3=red2,blue,blue2,green,green2,yellow,yellow2,10=teleport,11=clock,12=bookshelf,13=barrel,14=misc)
;  (bit 6); Unused
;  (bit 7) if set, treat coloured door as a normal door

  ; If bit 7 is set then convert to normal door
  BIT 7, A
  JR Z, skip1
  ; Reset bits 3 to 5 to convert to door or door2 (and also 7 while we are at it)
  AND %01000111
LOCAL skip1
skip1:

  ; Save Direction and Type as two separate items
  LD B, A
  AND 3
  LD (IX+8), A  ; (IX+8)=DoorDir
  LD A, B
  RR A
  RR A
  AND 15
  LD (IX+9), A  ; (IX+9)=DoorType2

  INC HL
  LD A, (HL)        ; Read RoomNo that this door leads to
  LD (IX+10), A     ; (IX+10) = DoorNextRoom

  POP IX
  END ASM
END SUB

FUNCTION FASTCALL MapGetDoorType() AS UBYTE
  ASM
  LD A, (DoorType2)
  END ASM
END FUNCTION

FUNCTION FASTCALL MapGetDoorDir() AS UBYTE
  ASM
  LD A, (DoorDir)
  END ASM
END FUNCTION

FUNCTION FASTCALL MapGetDoorNextRoom() AS UBYTE
  ASM
  LD A, (DoorNextRoom)
  END ASM
END FUNCTION









' --------------------------------------------------------
'   DOOR BUFFER
' --------------------------------------------------------


' Each array entry contains (Y,X) position and Door No that we should go to
' doorbufferCount is count of entries
DIM doorbufferCount AS UBYTE = 0
DIM doorbuffer(32,3) AS UBYTE		' Up to 32 character squares

SUB ResetDoorBuffer()
  doorbufferCount = 0
END SUB

SUB AddDoorBuffer(sy AS UBYTE, sx AS UBYTE, room AS UBYTE)
  doorbuffer(doorbufferCount, 0) = sy
  doorbuffer(doorbufferCount, 1) = sx
  doorbuffer(doorbufferCount, 2) = room
  doorbufferCount = doorbufferCount + 1
END SUB

FUNCTION GetNextRoomFromDoorBuffer(y AS UBYTE, x AS UBYTE) AS UBYTE
  DIM n AS UBYTE
  FOR n = 0 TO doorbufferCount-1
    IF(doorbuffer(n, 0) = y AND doorbuffer(n, 1) = x) THEN
      RETURN doorbuffer(n, 2)
    END IF
  NEXT n
  PRINT AT 0,0;"DoorBuffer (";y;",";x;") not found!"
  PrintDoorBuffer()
  STOP
END FUNCTION

SUB PrintDoorBuffer()
  DIM n AS UBYTE
  FOR n = 0 TO doorbufferCount-1
    PRINT "(";doorbuffer(n, 0);",";doorbuffer(n, 1);")=>";doorbuffer(n, 2)
  NEXT n
END SUB







' --------------------------------------------------------
'   OBJECTS
' --------------------------------------------------------

  ' Which room these keys are in.  When user has the key, the value is 255
  DIM ObjLoc(OBJECT_COUNT) AS UBYTE  ' 20 objects
  DIM ObjName$(OBJECT_COUNT)
  ObjName$(BLUE_KEY) = "Blue Key"
  ObjName$(RED_KEY) = "Red Key"
  ObjName$(GREEN_KEY) = "Green Key"
  ObjName$(YELLOW_KEY) = "Yellow Key"
  ObjName$(ACG1_KEY) = "AC Key"
  ObjName$(ACG2_KEY) = "G Key"
  ObjName$(ACG3_KEY) = "Handle Key"

  SUB InitialiseObjects()
    DIM n AS UBYTE
    FOR n = LBOUND(ObjLoc) TO UBOUND(ObjLoc)
      ObjLoc(n) = 0
    NEXT n

    ' 8 possible options: http://www.the-tipshop.co.uk/maps/a/AticAtac/AticAtac_MikeMyers.png
    DIM BluKeyLocs(7)  AS UBYTE => { 125, 117, 142, 125, 117, 142, 125, 117 }   ' Blue key always is at caverns. There are three possible places although the most common ones are one screen with a skeleton on left wall and a screen with trunks -and no bookcases
    DIM RedKeyLocs(7)  AS UBYTE => {  31,  02,  06,  27,  52,  37,  34,  54 } ' There are 8 possible places for red key. It always appear in the same floor where you found green key .
    DIM GrnKeyLocs(7)  AS UBYTE => {  24,  23,  22,  20,  49,  48,  47,  44 }    ' There are 8 possible places for green key. They can appear in ground floor (four possible places) or 1st floor (four possible places)
    DIM Acg1KeyLocs(7) AS UBYTE => {  73,  54,  35,  59,  57,  58, 114,  06 }  '
    DIM Acg2KeyLocs(7) AS UBYTE => { 134, 130, 124, 148,  54,  56,  01,  15 }
    DIM Acg3KeyLocs(7) AS UBYTE => {  91,  88,  91,  88,  91,  88,  91,  88 }   ' The Handle is always at attic. There are two possible places where you can find it.

    Dim r AS UBYTE = INT(RND() * 8)
    ObjLoc(BLUE_KEY)   = BluKeyLocs(r)
    ObjLoc(RED_KEY)    = RedKeyLocs(r)
    ObjLoc(GREEN_KEY)  = GrnKeyLocs(r)
    ObjLoc(YELLOW_KEY) = 68
    ObjLoc(ACG1_KEY)   = Acg1KeyLocs(r)
    ObjLoc(ACG2_KEY)   = Acg2KeyLocs(r)
    ObjLoc(ACG3_KEY)   = Acg3KeyLocs(r)

  END SUB

  SUB DrawObjects(roomno AS UBYTE)
    DIM py AS UBYTE = 5
    DIM n AS UBYTE
    FOR n = LBOUND(ObjLoc) TO UBOUND(ObjLoc)
      IF(ObjLoc(n) = roomno) THEN
        printat42(py, 12)
        print42(ObjName(n))
        ObjLoc(n) = 255 ' Collect
        BEEP .1, n
      END IF
      IF(ObjLoc(n) = 255) THEN
        printat42(py, 33)
        print42(ObjName(n))
        py = py + 1
      END IF
    NEXT n
  END SUB

  FUNCTION GotObject(n AS UBYTE) AS UBYTE
    IF(ObjLoc(n) = 255) THEN
      RETURN 1
    ELSE
      RETURN 0
    END IF
  END FUNCTION

  FUNCTION HasACG() AS UBYTE
    IF(ObjLoc(ACG1_KEY) = 255 AND ObjLoc(ACG2_KEY) = 255 AND ObjLoc(ACG3_KEY) = 255) THEN
      RETURN 1
    ELSE
      RETURN 0
    END IF
  END FUNCTION










ASM
                  DEFB  255,255,255,255
RoomPointer:	    DEFW	0   ; IX+0,IX+1
RoomType:	        DEFB	0   ; IX+2
RoomColour:  	    DEFB	0   ; IX+3
RoomDoorCount:  	DEFB	0   ; IX+4
; Current Door
DoorPointer:	    DEFW	0   ; IX+5,IX+6
DoorType:         DEFB  0   ; IX+7
DoorDir:          DEFB  0   ; IX+8  ; 00=North; 01=East; 02=South; 03=West or ALT
DoorType2:        DEFB  0   ; IX+9  ; Original type without door data
DoorNextRoom:     DEFB  0   ; IX+10
END ASM


